C
C***********************************************************************
C
C     FUNCTION TRIANG
C
C     PURPOSE
C       COMPUTE THE VERTICAL GRAVITY ATTRACTION OF A TRIANGULAR PRISM
C       WITH AN INCLINED TOP.
C
C     USAGE
C       TRIANG(X0,Y0,Z0)
C
C     DESCRIPTION OF PARAMETERS
C       X0    -INPUT, EASTING (X) COORDINATE OF THE POINTS DEFINING THE
C              TRIANGLE.
C       Y0    _INPUT, NORTHING (Y) COORDINATE OF THE POINTS DEFINING THE
C              TRIANGLE.
C       Z0    -INPUT, ELEVATION (Z) COORDINATE OF THE POINTS DEFINING
C              THE TRIANGLE.
C
C     REMARKS
C       THE FINAL GRAVITY ATTRACTION IS GIVEN BY G*P*TRIANG,
C       WHERE:
C             G= GRAVITATIONNAL CONSTANT.
C             P= DENSITY (G./CC.).
C       THE INPUT ORDER OF THE POINTS DEFINING THE TRIANGLE IS NOT
C       IMPORTANT.
C
C     SUBROUTINE USED
C       NONE
C
C     FUNCTIONS USED
C       NONE
C
C     METHOD
C       AN EXACT FORMULATION IS USED TO COMPUTE THE VERTICAL GRAVITY
C       ATTRACTION OF THE TRIANGULAR PRISM.
C
C     REFERENCE
C        WOODWARD, O. J. (1975) THE GRAVITATIONAL ATTRACTION OF VERTICAL
C        TRIANGULAR PRISM. GEOPHYSICAL PROSPECTING 23, PP. 526-532.
C
C     TIME REQUIRED
C
C
C     SIZE
C
C
C     PROGRAMMER
C       REMI FERLAND
C
C     DATE WRITTEN
C       FEBUARY 1982.
C
C***********************************************************************
C
      FUNCTION TRIANG(X0,Y0,Z0)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C      REAL*4 TRIANG,X0(3),Y0(3),Z0(3)
      DIMENSION X0(3),Y0(3),Z0(3)
      DIMENSION X(4),Y(4),Z(4)
      SCALE=1.0D0
      DO 3 I=1,3
        X(I)=X0(I)/SCALE
        Y(I)=Y0(I)/SCALE
        Z(I)=Z0(I)/SCALE
    3 CONTINUE
C
C     DETERMINATION OF THE EQUATION OF THE INCLINED PLANE Z=A1*X+A2*Y+A3
C
      SUMT=0.0
      IF(Z(1).EQ.0.0.AND.Z(2).EQ.0.0.AND.Z(3).EQ.0.0)GO TO 20
      A=X(2)*Y(3)-X(3)*Y(2)
      B=X(3)*Y(1)-X(1)*Y(3)
      C=X(1)*Y(2)-X(2)*Y(1)
      D=A+B+C
C
C     D=DETERMINANT WHEN=0 THEN,NO TRIANGLE POSSIBLE
C
      IF(ABS(D).LT.1.E-6)GO TO 20
C
C     A1,A2,A3 => COEFICIENTS OF THE INCLINED PLANE
C
      A1=(Z(1)*(Y(2)-Y(3))+Z(2)*(Y(3)-Y(1))+Z(3)*(Y(1)-Y(2)))/D
      A22=(Z(1)*(X(3)-X(2))+Z(2)*(X(1)-X(3))+Z(3)*(X(2)-X(1)))/D
      A3=(Z(1)*A+Z(2)*B+Z(3)*C)/D
C     WRITE(3,100)X(1),Y(1),Z(1)
C     WRITE(3,100)X(2),Y(2),Z(2)
C     WRITE(3,100)X(3),Y(3),Z(3)
C     WRITE(3,100)A1,A22,A3
      IF(ABS(A1).LT.1.E-6.AND.ABS(A22).LT.1.E-6.AND.ABS(A3).LT.1.E-6)GO
     &TO 20
C
C     COMPUTATION OF THE GRAVITATIONNAL ATTRACTION
C
      X(4)=X(1)
      Y(4)=Y(1)
      Z(4)=Z(1)
      DO 11 I=1,3
        SUM=0.0
        IF(ABS(X(I)-X(I+1)).LT.1.E-5)GO TO 10
C
C       Y=0.0
C
C       DETERMINATION OF THE COEFICIENTS A,B OF THE SIDE OF THE
C       TRIANGLE (Y=A*X+B).
C
        A=(Y(I+1)-Y(I))/(X(I+1)-X(I))
        B=Y(I)-A*X(I)
C       WRITE(3,100)A,B
        IF(ABS(A).LT.1.E-6)A=0.0
        IF(ABS(B).LT.1.E-6)B=0.0
        IF(ABS(A1).LT.1.E-6)A1=0.0
        IF(ABS(A22).LT.1.E-6)A22=0.0
        IF(ABS(A3).LT.1.E-6)A3=0.0
        A2=A22
        M=1
        IF(B.GE.0.0)GO TO 15
        B=-1.*B
        A=-1.*A
        A2=-1.*A22
        M=-1
   15   IF(ABS(A).LT.1.E-6.AND.ABS(B).LT.1.E-6)GO TO 10
        IF(ABS(A).LT.1.E-6.AND.ABS(A1).LT.1.E-6.AND.ABS(A3).LT.1.E-6)GO
     &  TO 7
        IF(ABS(A).LT.1.E-6.AND.ABS(A2).LT.1.E-6.AND.ABS(A3).LT.1.E-6)GO
     &  TO 8
        IF(ABS(B).LT.1.E-6)GO TO 2
        E=A*A+1.0
C       G=K=M
        G=B/E
C       TI=TAN(TETA(I))
C       TJ=TAN(TETA(I+1))
        TI=E*X(I)/B+A
        TJ=E*X(I+1)/B+A
        T1=ATAN2(E*X(I)+A*B,B)
        T2=ATAN2(E*X(I+1)+A*B,B)
C       C=N
        E=SQRT(E)
        C=B/E
C       F=L
        F=A*G
        SUM=TJ*DLOG(ABS(F*TJ+G+C/DCOS(T2)))-TI*DLOG(ABS(F*TI+G+
     &  C/DCOS(T1)))
        TI=TAN(T1/2.)
        TJ=TAN(T2/2.)
        SUM=SUM+G/(C+F)*DLOG((1.-TI)/(1.-TJ))+G/(C-F)*DLOG((1.+TJ)/(1.+T
     &  I))
        SUM=SUM+F*G/(C*C-F*F)*DLOG((((C-G)*TI+2.*F)*TI+C+G)/(((C-G)*TJ+2
     &  .*F)*TJ+C+G))
        SUM=G*(SUM+2.*(TI/(1.-TI*TI)-TJ/(1.-TJ*TJ)))
        IF(X(I).NE.0.0)SUM=SUM-X(I)*(1.-DLOG(ABS(X(I))))
        IF(X(I+1).NE.0.0)SUM=SUM+X(I+1)*(1.-DLOG(ABS(X(I+1))))
        GO TO 4
    2   IF(ABS(A3).LT.1.E-6)GO TO 5
        SUM=(X(I+1)-X(I))*DLOG(A+SQRT(1.+A*A))
C
C       Y=M*X + N
C
    4   H=1.+A2*A2
C       C=ALPHA
        C=A1*A2+H*A
C       D=BETA
        D=A2*A3+H*B
C       E=GAMMA
        E=H*(A*A+1.+(A2*A+A1)**2)
C       F=DELTA
        F=H*((A2*A+A1)*(A2*B+A3)+A*B)
C       G=ETA
        G=H*(B*B+(A2*B+A3)**2)
        T1=G*E-F*F
        T11=SQRT(T1)
C       A=U
        A=T11*C/E
C       BB=V
        BB=SQRT(T1/E)
C       D=W
        D=D-C*F/E
C       TI=TAN(TETA(I))
C       TJ=TAN(TETA(I+1))
        TI=(E*X(I)+F)/T11
        TJ=(E*X(I+1)+F)/T11
        T1=ATAN2(E*X(I)+F,T11)
        T2=ATAN2(E*X(I+1)+F,T11)
        SU=TJ*DLOG(ABS(A*TJ+D+BB/COS(T2)))-TI*DLOG(ABS(A*TI+D+BB/COS(T1)
     &  ))
        TI=TAN(T1/2.)
        TJ=TAN(T2/2.)
        T1=BB*BB-A*A
        T2=SQRT(ABS(T1-D*D))
        SU=SU+D/(A+BB)*DLOG((1.-TI)/(1.-TJ))-D/(BB-A)*DLOG((1.+TI)/(1.+T
     &  J))
        SU=SU+2.*(TI/(1.-TI*TI)-TJ/(1.-TJ*TJ))
        SU=SU+A*D/T1*DLOG(((BB-D)*TI*TI+2.*A*TI+BB+D)/((BB-D)*TJ*TJ+2.*A
     &  *TJ+BB+D))
        IF(T2.GT.1.E-6)SU=SU-2.*BB*T2/T1*(ATAN2(((BB-D)*TI+A),T2)-ATAN2(
     &  ((BB-D)*TJ+A),T2))
        SU=SU*T11/E
        IF(ABS(A3).LT.1.E-6)GO TO 9
        IF(ABS(A1).LT.1.E-6.AND.ABS(A2).LT.1.E-6)GO TO 6
        C=A1*A2
        D=A2*A3
        E=H*(1.+A1*A1)
        F=H*A1*A3
        G=H*A3*A3
        T1=G*E-F*F
        T11=SQRT(T1)
        A=T11*C/E
        BB=SQRT(T1/E)
        D=D-C*F/E
        TI=(E*X(I)+F)/T11
        TJ=(E*X(I+1)+F)/T11
        T1=ATAN2(E*X(I)+F,T11)
        T2=ATAN2(E*X(I+1)+F,T11)
        SU1=TJ*DLOG(A*TJ+D+BB/COS(T2))-TI*DLOG(A*TI+D+BB/COS(T1))
        TI=TAN(T1/2.)
        TJ=TAN(T2/2.)
        T1=BB*BB-A*A
        T2=SQRT(ABS(T1-D*D))
        SU1=SU1+D/(A+BB)*DLOG((1.-TI)/(1.-TJ))-D/(BB-A)*DLOG((1.+TI)/(1.
     &  +TJ))
        SU1=SU1+2.*(TI/(1.-TI*TI)-TJ/(1.-TJ*TJ))
        SU1=SU1+A*D/T1*DLOG((((BB-D)*TI+2.*A)*TI+BB+D)/(((BB-D)*TJ+2.*A)
     &  *TJ+BB+D))
        SU1=SU1-2.*BB*T2/T1*(ATAN2(((BB-D)*TI+A),T2)-ATAN2(((BB-D)*TJ+A)
     &  ,T2))
        IF(ABS(B).LT.1.E-6.AND.(X(I)+X(I+1)).LT.0.0)GO TO 30
        SUM=SUM-(SU-SU1*T11/E)/SQRT(H)
        GO TO 10
   30   SUM=-1.*(SUM+(SU-SU1*T11/E)/SQRT(H))
        GO TO 10
    9   D=SQRT((1.+A1*A1)*H)
        E=A1*A2
        IF(X(I+1).NE.0.0)SU=SU-X(I+1)*(DLOG(ABS((E+SIGN(D,X(I+1)))*X(I+1
     &  )))-1.)
        IF(X(I).NE.0.0)SU=SU+X(I)*(DLOG(ABS((E+SIGN(D,X(I)))*X(I)))-1.)
        SUM=SUM-SU/SQRT(H)
        GO TO 10
    6   C=A3*A3
        SU1=0.0
        IF(X(I+1).NE.0.0)SU1=X(I+1)*(DLOG(SQRT(X(I+1)*X(I+1)+C))-1.)+A3*
     &  ATAN2(X(I+1),A3)
        IF(X(I).NE.0.0)SU1=SU1-X(I)*(DLOG(SQRT(X(I)*X(I)+C))-1.)-A3*ATAN
     &  2(X(I),A3)
        IF(ABS(B).LT.1.E-6.AND.(X(I)+X(I+1)).LT.0.0)GO TO 35
        SUM=SUM-SU+SU1
        GO TO 10
   35   SUM=SU1-SU-SUM
        GO TO 10
    5   D=1.+A2*A2
        C=A1*A2+D*A
        E=SQRT(D*(A*A+1.+(A2*A+A1)**2))
        T1=SQRT(1.+A*A)
        T2=A1*A2+SQRT(D*(1.+A1*A1))
        D=SQRT(D)
C       WRITE(3,100)D,C,E,T1,T2,X(I),X(I+1)
        SUM=SIGN(1.D0,(X(I)+X(I+1)))*(X(I+1)-X(I))*(DLOG(A+T1)-
     &  DLOG((C+E)/T2)/D)
        GO TO 10
    7   C=B*B
        D=1.+A2*A2
        E=SQRT(D)
        F=C*D
        T1=SQRT(X(I)*X(I)+C)
        T2=SQRT(X(I+1)*X(I+1)+C)
        TI=SQRT(X(I)*X(I)+F)
        TJ=SQRT(X(I+1)*X(I+1)+F)
        F=SQRT(F)
        SUM=B*DLOG((X(I+1)+T2)*(X(I)+TI)/((X(I)+T1)*(X(I+1)+TJ)))
        IF(X(I+1).NE.0.0)SUM=SUM+X(I+1)*(DLOG(ABS((T2+B)/X(I+1)))-DLOG(A
     &  BS((TJ+F)/X(I+1)))/E)
        IF(X(I).NE.0.0)SUM=SUM-X(I)*(DLOG(ABS((T1+B)/X(I)))-DLOG(ABS((TI
     &  +F)/X(I)))/E)
        GO TO 10
    8   C=B*B
        D=1.+A1*A1
        E=SQRT(D)
        T1=SQRT(X(I)*X(I)+C)
        T2=SQRT(X(I+1)*X(I+1)+C)
        TI=SQRT(X(I)*X(I)*D+C)
        TJ=SQRT(X(I+1)*X(I+1)*D+C)
        SUM=B*(DLOG((T2+X(I+1))/(T1+X(I)))-DLOG((TJ+E*X(I+1))/(TI+E*X(I)
     &  ))/E)
        IF(X(I+1).NE.0.0)SUM=SUM+X(I+1)*DLOG((T2+B)*E/(TJ+B))
        IF(X(I).NE.0.0)SUM=SUM-X(I)*DLOG((T1+B)*E/(TI+B))
   10   CONTINUE
  100   FORMAT(F15.2)
        SUMT=SUMT+M*SUM
   11 CONTINUE
C     WRITE(3,100)ABS(SUMT)
   20 CONTINUE
C     CLOSE(3)
      TRIANG=ABS(SUMT*SCALE)
      RETURN
      END

